pub struct Preceded<A, AVal, B, const CHECK: bool = false> {
pub a: A,
pub b: B,
pub a_val: AVal,
}Expand description
Parsing semantics: like (A, B), but discards the value parsed by A and returns only the value parsed by B.
Serialization semantics: reuses a_val as the serialized witness for A, then serializes B.
When CHECK is false, parsing is malleable in the discarded prefix unless A admits a unique consistent value.
When CHECK is true, parsing additionally checks that the parsed prefix equals a_val.
Fields
a: Ab: Ba_val: AValTrait Implementations
Sourceimpl<A: Clone, AVal: Clone, B: Clone, const CHECK: bool> Clone for Preceded<A, AVal, B, CHECK>
impl<A: Clone, AVal: Clone, B: Clone, const CHECK: bool> Clone for Preceded<A, AVal, B, CHECK>
Sourceexec fn clone(&self) -> cloned : Self
exec fn clone(&self) -> cloned : Self
ensures
call_ensures(A::clone, (&self.a,), cloned.a),call_ensures(B::clone, (&self.b,), cloned.b),call_ensures(AVal::clone, (&self.a_val,), cloned.a_val),1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<A, B, const CHECK: bool> Consistency for Preceded<A, A::Val, B, CHECK>where
A: Consistency,
B: Consistency,
impl<A, B, const CHECK: bool> Consistency for Preceded<A, A::Val, B, CHECK>where
A: Consistency,
B: Consistency,
Sourceopen spec fn consistent(&self, v: Self::Val) -> bool
open spec fn consistent(&self, v: Self::Val) -> bool
{
let fmt = preceded::<_, _, _, _, CHECK>(self.a, self.b, self.a_val);
fmt.consistent(v)
}Sourcetype Val = <B as Consistency>::Val
type Val = <B as Consistency>::Val
The type of values whose consistency is being checked.
Sourceimpl<A, B, const CHECK: bool> EquivSerializersGeneral for Preceded<A, A::SValue, B, CHECK>
impl<A, B, const CHECK: bool> EquivSerializersGeneral for Preceded<A, A::SValue, B, CHECK>
Sourceopen spec fn equiv_general_inv(&self) -> bool
open spec fn equiv_general_inv(&self) -> bool
{
let fmt = preceded::<_, _, _, _, CHECK>(self.a, self.b, self.a_val);
fmt.equiv_general_inv()
}Sourceproof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
proof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
Sourceimpl<A, B, const CHECK: bool> GoodSerializer for Preceded<A, A::SVal, B, CHECK>where
A: GoodSerializer,
B: GoodSerializer,
impl<A, B, const CHECK: bool> GoodSerializer for Preceded<A, A::SVal, B, CHECK>where
A: GoodSerializer,
B: GoodSerializer,
Sourceopen spec fn serialize_inv(&self) -> bool
open spec fn serialize_inv(&self) -> bool
{
let fmt = preceded::<_, _, _, _, CHECK>(self.a, self.b, self.a_val);
fmt.serialize_inv()
}Sourceproof fn lemma_serialize_len(&self, v: Self::SVal)
proof fn lemma_serialize_len(&self, v: Self::SVal)
Sourceimpl<A, B, const CHECK: bool> MinMaxByteLen for Preceded<A, A::T, B, CHECK>where
A: MinMaxByteLen,
B: MinMaxByteLen,
impl<A, B, const CHECK: bool> MinMaxByteLen for Preceded<A, A::T, B, CHECK>where
A: MinMaxByteLen,
B: MinMaxByteLen,
Sourceimpl<A, B, const CHECK: bool> NoLookAhead for Preceded<A, A::PVal, B, CHECK>where
A: NoLookAhead,
B: NoLookAhead,
impl<A, B, const CHECK: bool> NoLookAhead for Preceded<A, A::PVal, B, CHECK>where
A: NoLookAhead,
B: NoLookAhead,
Sourceopen spec fn no_lookahead_inv(&self) -> bool
open spec fn no_lookahead_inv(&self) -> bool
{
let fmt = preceded::<_, _, _, _, CHECK>(self.a, self.b, self.a_val);
fmt.no_lookahead_inv()
}Sourceproof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
proof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
Sourceimpl<A, B> NonMalleable for Preceded<A, A::PVal, B, false>
impl<A, B> NonMalleable for Preceded<A, A::PVal, B, false>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{
&&& Pair(self.a, self.b).sound_inv()
&&& Pair(self.a, self.b).nonmal_inv()
}Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<A, B> NonMalleable for Preceded<A, A::PVal, B, true>
impl<A, B> NonMalleable for Preceded<A, A::PVal, B, true>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{
&&& Pair(self.a, self.b).sound_inv()
&&& Pair(self.a, self.b).nonmal_inv()
}Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<A, B, const CHECK: bool> NonTailFmt for Preceded<A, A::SValue, B, CHECK>where
A: NonTailFmt,
B: NonTailFmt,
impl<A, B, const CHECK: bool> NonTailFmt for Preceded<A, A::SValue, B, CHECK>where
A: NonTailFmt,
B: NonTailFmt,
Sourceopen spec fn serialize_dps_inv(&self) -> bool
open spec fn serialize_dps_inv(&self) -> bool
{
let fmt = preceded::<_, _, _, _, CHECK>(self.a, self.b, self.a_val);
fmt.serialize_dps_inv()
}Sourceproof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceproof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceimpl<I, A, AVal, B> Parser<I> for Preceded<A, AVal, B, false>where
I: InputBuf,
A: Parser<I, PT = AVal> + SafeParser<PVal = AVal>,
B: Parser<I> + SafeParser,
AVal: DeepView<V = AVal>,
impl<I, A, AVal, B> Parser<I> for Preceded<A, AVal, B, false>where
I: InputBuf,
A: Parser<I, PT = AVal> + SafeParser<PVal = AVal>,
B: Parser<I> + SafeParser,
AVal: DeepView<V = AVal>,
Sourceimpl<I, A, AVal, B> Parser<I> for Preceded<A, AVal, B, true>where
I: InputBuf,
A: Parser<I, PT = AVal> + SafeParser<PVal = AVal>,
B: Parser<I> + SafeParser,
AVal: DeepView<V = AVal> + PartialEq + Structural,
impl<I, A, AVal, B> Parser<I> for Preceded<A, AVal, B, true>where
I: InputBuf,
A: Parser<I, PT = AVal> + SafeParser<PVal = AVal>,
B: Parser<I> + SafeParser,
AVal: DeepView<V = AVal> + PartialEq + Structural,
Sourceimpl<A, B> Productive for Preceded<A, A::PVal, B, false>where
A: Productive,
B: Productive,
impl<A, B> Productive for Preceded<A, A::PVal, B, false>where
A: Productive,
B: Productive,
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{ preceded::<_, _, _, _, false>(self.a, self.b, self.a_val).productive_inv() }Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<A, B> Productive for Preceded<A, A::PVal, B, true>where
A: Productive,
B: Productive,
impl<A, B> Productive for Preceded<A, A::PVal, B, true>where
A: Productive,
B: Productive,
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{ preceded::<_, _, _, _, true>(self.a, self.b, self.a_val).productive_inv() }Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<A, B, const CHECK: bool> SPRoundTripDps for Preceded<A, A::SValue, B, CHECK>
impl<A, B, const CHECK: bool> SPRoundTripDps for Preceded<A, A::SValue, B, CHECK>
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{ Pair(self.a, self.b).unambiguous() }Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
Sourceimpl<A, B, const CHECK: bool> SafeParser for Preceded<A, A::PVal, B, CHECK>where
A: SafeParser,
B: SafeParser,
impl<A, B, const CHECK: bool> SafeParser for Preceded<A, A::PVal, B, CHECK>where
A: SafeParser,
B: SafeParser,
Sourceimpl<Output: OutputBuf, A, AVal, B, T, const CHECK: bool> Serializer<Output, T> for Preceded<A, AVal, B, CHECK>where
AVal: DeepView<V = AVal>,
T: DeepView,
A: Serializer<Output, AVal>,
B: Serializer<Output, T>,
impl<Output: OutputBuf, A, AVal, B, T, const CHECK: bool> Serializer<Output, T> for Preceded<A, AVal, B, CHECK>where
AVal: DeepView<V = AVal>,
T: DeepView,
A: Serializer<Output, AVal>,
B: Serializer<Output, T>,
Sourceopen spec fn exec_inv(&self) -> bool
open spec fn exec_inv(&self) -> bool
{
&&& self.a.exec_inv()
&&& self.b.exec_inv()
&&& forall |v: AVal| v.deep_view() == v
}Sourceexec fn serialize_into(&self, v: &T, obuf: &mut Output)
exec fn serialize_into(&self, v: &T, obuf: &mut Output)
Sourceimpl<A, B> SoundParser for Preceded<A, A::PVal, B, false>
impl<A, B> SoundParser for Preceded<A, A::PVal, B, false>
Sourceimpl<A, B> SoundParser for Preceded<A, A::PVal, B, true>where
A: SoundParser,
B: SoundParser,
impl<A, B> SoundParser for Preceded<A, A::PVal, B, true>where
A: SoundParser,
B: SoundParser,
Sourceimpl<A, B, const CHECK: bool> SpecByteLen for Preceded<A, A::T, B, CHECK>where
A: SpecByteLen,
B: SpecByteLen,
impl<A, B, const CHECK: bool> SpecByteLen for Preceded<A, A::T, B, CHECK>where
A: SpecByteLen,
B: SpecByteLen,
Sourceimpl<A, B, const CHECK: bool> SpecParser for Preceded<A, A::PVal, B, CHECK>where
A: SpecParser,
B: SpecParser,
impl<A, B, const CHECK: bool> SpecParser for Preceded<A, A::PVal, B, CHECK>where
A: SpecParser,
B: SpecParser,
Sourceimpl<A, B, const CHECK: bool> SpecSerializer for Preceded<A, A::SVal, B, CHECK>where
A: SpecSerializer,
B: SpecSerializer,
impl<A, B, const CHECK: bool> SpecSerializer for Preceded<A, A::SVal, B, CHECK>where
A: SpecSerializer,
B: SpecSerializer,
Sourceopen spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
{
let fmt = preceded::<_, _, _, _, CHECK>(self.a, self.b, self.a_val);
fmt.spec_serialize(v)
}Sourcetype SVal = <B as SpecSerializer>::SVal
type SVal = <B as SpecSerializer>::SVal
The type of values to be serialized.
Sourceimpl<A, B, const CHECK: bool> SpecSerializerDps for Preceded<A, A::SValue, B, CHECK>where
A: SpecSerializerDps,
B: SpecSerializerDps,
impl<A, B, const CHECK: bool> SpecSerializerDps for Preceded<A, A::SValue, B, CHECK>where
A: SpecSerializerDps,
B: SpecSerializerDps,
Sourceimpl<A, B, const CHECK: bool> StaticByteLen for Preceded<A, A::T, B, CHECK>where
A: StaticByteLen,
B: StaticByteLen,
impl<A, B, const CHECK: bool> StaticByteLen for Preceded<A, A::T, B, CHECK>where
A: StaticByteLen,
B: StaticByteLen,
Sourceopen spec fn static_byte_len() -> nat
open spec fn static_byte_len() -> nat
{ <Pair<A, B> as StaticByteLen>::static_byte_len() }Sourceproof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
Sourceimpl<A, B, const CHECK: bool> ValueByteLen for Preceded<A, A::T, B, CHECK>where
A: StaticByteLen,
B: ValueByteLen,
impl<A, B, const CHECK: bool> ValueByteLen for Preceded<A, A::T, B, CHECK>where
A: StaticByteLen,
B: ValueByteLen,
Sourceopen spec fn value_byte_len(v: Self::T) -> nat
open spec fn value_byte_len(v: Self::T) -> nat
{ A::static_byte_len() + B::value_byte_len(v) }Sourceproof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
impl<A: Copy, AVal: Copy, B: Copy, const CHECK: bool> Copy for Preceded<A, AVal, B, CHECK>